==> glib-web-0.4.28.tgz: package/components/mixins/table/import.js <==
export default {
  data: function () {
    return {
      importable: false,
      importSubmitUrl: null,
      importParamName: "row"
    }
  },
  methods: {
    initCsvImport() {
      const vm = this
      Utils.type.ifObject(this.spec.import, obj => {
        vm.importable = true
        vm.importSubmitUrl = obj.submitUrl
        vm.importParamName = obj.paramName
      })
    },
    submitRows(event, section) {
      const vm = this
      const keys = section.header.dataCells
      const rows = []
      section.dataRows.forEach((row, index) => {
        const cells = {}
        row.forEach((cell, index) => {
          const key = `${vm.importParamName}[${keys[index]}]`
          cells[key] = cell
        })
        rows.push(cells)
      })

      if (Utils.type.isString(this.importSubmitUrl)) {
        vm._submitEachRow(rows, event.target)
      }
    },
    _submitEachRow(rows, target) {
      const vm = this
      const row = rows.shift()
      if (row) {
        const data = {
          url: this.importSubmitUrl,
          formData: row,
        }
        Utils.http.execute(data, 'POST', vm, (response) => {
          Action.execute(response['onResponse'], target, vm)
          vm._submitEachRow(rows, target)
        })
      }
    },
    loadFile(event, section) {
      const reader = new FileReader()
      reader.readAsText(event.target.files[0])
      reader.onload = e => {
        const rows = this._parseCsv(e.target.result)
        section.header.dataCells = rows.shift()
        section.dataRows = rows
      }
    },
    _parseCsv: function(csvString) {
      // https://stackoverflow.com/a/41563966/9970813
      let prevLetter = '', row = [''], result = [row], columnIndex = 0, rowIndex = 0, canSplit = true, letter;
      for(let i = 0; i <= csvString.length; ++i){
        letter = csvString[i]

        if ('"' === letter) {
          if (canSplit && letter === prevLetter){
            row[columnIndex] += letter;
          }
          canSplit = !canSplit;
        } else if (',' === letter && canSplit) {
          letter = row[++columnIndex] = '';
        } else if ('\n' === letter && canSplit) {
            if ('\r' === prevLetter) {
              row[columnIndex] = row[columnIndex].slice(0, -1);
            }
            row = result[++rowIndex] = [letter = ''];
            columnIndex = 0;
        } else if (Utils.type.isString(letter)) {
          row[columnIndex] += letter;
        }
        prevLetter = letter;
      }

      // Remove any blank rows
      result = result.filter(r => {
        return r[0] !== "undefined"
      });
      
      return result;
    }
  }
}

==> glib-web-0.4.39.tgz: package/components/mixins/table/import.js <==
export default {
  data: function () {
    return {
      importable: false,
      importSubmitUrl: null,
      importParamName: "row"
    }
  },
  methods: {
    initCsvImport() {
      const vm = this
      Utils.type.ifObject(this.spec.import, obj => {
        vm.importable = true
        vm.importSubmitUrl = obj.submitUrl
        vm.importParamName = obj.paramName
      })
    },
    submitRows(event, section) {
      const vm = this
      const keys = section.header.dataCells
      const rows = []
      section.dataRows.forEach((row, index) => {
        const cells = {}
        row.forEach((cell, index) => {
          const key = `${vm.importParamName}[${keys[index]}]`
          cells[key] = cell
        })
        rows.push(cells)
      })

      if (Utils.type.isString(this.importSubmitUrl)) {
        vm._submitEachRow(rows, event.target)
      }
    },
    _submitEachRow(rows, target) {
      const vm = this
      const row = rows.shift()
      if (row) {
        const data = {
          url: this.importSubmitUrl,
          formData: row,
        }
        Utils.http.execute(data, 'POST', vm, (response) => {
          Action.execute(response['onResponse'], target, vm)
          vm._submitEachRow(rows, target)
        })
      }
    },
    loadFile(event, section) {
      const reader = new FileReader()
      reader.readAsText(event.target.files[0])
      reader.onload = e => {
        const rows = this._parseCsv(e.target.result)
        section.header.dataCells = rows.shift()
        section.dataRows = rows
      }
    },
    _parseCsv: function(csvString) {
      // https://stackoverflow.com/a/41563966/9970813
      let prevLetter = '', row = [''], result = [row], columnIndex = 0, rowIndex = 0, canSplit = true, letter;
      for(let i = 0; i <= csvString.length; ++i){
        letter = csvString[i]

        if ('"' === letter) {
          if (canSplit && letter === prevLetter){
            row[columnIndex] += letter;
          }
          canSplit = !canSplit;
        } else if (',' === letter && canSplit) {
          letter = row[++columnIndex] = '';
        } else if ('\n' === letter && canSplit) {
            if ('\r' === prevLetter) {
              row[columnIndex] = row[columnIndex].slice(0, -1);
            }
            row = result[++rowIndex] = [letter = ''];
            columnIndex = 0;
        } else if (Utils.type.isString(letter)) {
          row[columnIndex] += letter;
        }
        prevLetter = letter;
      }

      // Remove any blank rows
      result = result.filter(r => {
        return r[0] !== ""
      });
      // result = result.filter(r => {
      //   return r[0] !== "undefined"
      // });

      return result;
    }
  }
}
